----------------------------------------------------------------------------------
-- Company: 
-- Engineer: 
-- 
-- Create Date: 23.05.2022 12:00:57
-- Design Name: 
-- Module Name: FSM - Behavioral
-- Project Name: 
-- Target Devices: 
-- Tool Versions: 
-- Description: 
-- 
-- Dependencies: 
-- 
-- Revision:
-- Revision 0.01 - File Created
-- Additional Comments:
-- 
----------------------------------------------------------------------------------

library IEEE;
use IEEE.STD_LOGIC_1164.ALL;
use IEEE.STD_LOGIC_UNSIGNED.ALL;

-- Uncomment the following library declaration if using
-- arithmetic functions with Signed or Unsigned values
--use IEEE.NUMERIC_STD.ALL;

-- Uncomment the following library declaration if instantiating
-- any Xilinx leaf cells in this code.
--library UNISIM;
--use UNISIM.VComponents.all;

entity FSM is 
 generic (N: integer := 3); 
 port (
    rst          : in std_logic;
    clk          : in std_logic;
    play         : in std_logic;
    state_out    : out std_logic_vector(N-1 downto 0);
    
    error_inject : in std_logic;
    error_data   : in std_logic_vector(1 downto 0);
    alarm        : out std_logic
);
end FSM;

architecture Behavioral of FSM is

constant s0 : std_logic_vector(N-1 downto 0) := "000";
constant s1 : std_logic_vector(N-1 downto 0) := "001";
constant s2 : std_logic_vector(N-1 downto 0) := "010";
constant s3 : std_logic_vector(N-1 downto 0) := "011"; -- UNUSED STATE
constant s4 : std_logic_vector(N-1 downto 0) := "100";
constant s5 : std_logic_vector(N-1 downto 0) := "101";
constant s6 : std_logic_vector(N-1 downto 0) := "110";
constant s7 : std_logic_vector(N-1 downto 0) := "111"; 

signal current_state : std_logic_vector(N-1 downto 0); 
signal next_state    : std_logic_vector(N-1 downto 0); 

begin

-- State sequence: s0-s1-s2-s4-s5-s6-s7-s0-...
-- S3 state is unused (invalid)
next_state <= current_state when current_state = s3 else
              s4 when current_state = s2 else
              current_state + "001";
              
process(rst, clk)
begin 
  if rst = '1' then 
    current_state <= s0;

  elsif rising_edge(clk) then

    if play  = '1' then 
      current_state <= next_state;
    elsif error_inject = '1' then
      current_state(1 downto 0) <= current_state(1 downto 0) XOR error_data;
    end if;

  end if;
end process;

state_out <= current_state;

alarm <= '1' when (current_state = "011") else -- Invalid state 
         '0';
                 
end Behavioral;
