----------------------------------------------------------------------------------
-- Company: 
-- Engineer: 
-- 
-- Create Date: 23.05.2022 12:37:24
-- Design Name: 
-- Module Name: top_level - Behavioral
-- Project Name: 
-- Target Devices: 
-- Tool Versions: 
-- Description: 
-- 
-- Dependencies: 
-- 
-- Revision:
-- Revision 0.01 - File Created
-- Additional Comments:
-- 
----------------------------------------------------------------------------------


library IEEE;
use IEEE.STD_LOGIC_1164.ALL;

-- Uncomment the following library declaration if using
-- arithmetic functions with Signed or Unsigned values
--use IEEE.NUMERIC_STD.ALL;

-- Uncomment the following library declaration if instantiating
-- any Xilinx leaf cells in this code.
--library UNISIM;
--use UNISIM.VComponents.all;

entity top_level is
 port (
    rst          : in std_logic;
    clk          : in std_logic;
    play         : in std_logic;
    state_out    : out std_logic_vector(7 downto 0);
    
    error_inject : in std_logic;
    error_data   : in std_logic_vector(1 downto 0);
    alarm        : out std_logic
 );
 end top_level;

architecture Behavioral of top_level is

signal clk_1Hz        : std_logic;
signal error_inject_p : std_logic;

-- !!!!!!!!!!!!!!! ADD THE CORRECT VALUE FOR N !!!!!!!!!!!!!!
-- Binary encoding: 3, One-hot: 8, Hamming-3: 6
constant N : integer := 3;

begin

  -- Instantiate the frequency divider
  freq_div_u: 
  entity work.freq_div(Behavioral) 
    generic map(CLK_INPUT  => 125000000,		--Clock input frequency:  1250MHz
				CLK_OUTPUT => 1)				--Clock output frequency: 1Hz
	port map(clk_in  => clk,
			 clk_out => clk_1Hz);
	
  -- Instantiate the Pulse Generator for the "error_inject" signal
  pulse_gen_u: 
    entity work.pulse_gen(Behavioral) 
    port map(clk       => clk_1Hz,
			 pulse_in  => error_inject,
			 pulse_out => error_inject_p
			 );

  -- !!!!!!!!!!!!!!! Instantiate the Target FSM !!!!!!!!!!!!!!
  -- Options: FSM (Binary), FSM_Onehot, FSM_Hamming3
  FSM_u: 
  entity work.FSM(Behavioral)        -- Options: FSM(Behavioral), FSM_Onehot(Behavioral), FSM_Hamming3(Behavioral)
    generic map(N => N)                     -- Binary encoding: 3, One-hot: 8, Hamming-3: 6: 
	port map(rst          => rst,
             clk          => clk_1Hz,
             play         => play,
             state_out    => state_out(N-1 downto 0),
             error_inject => error_inject_p,
             error_data   => error_data,
             alarm        => alarm
            );
   
   -- ADD THIS STATEMENT ONLY FOR binary encoding and Hamming-3
   state_out (7 downto N) <= (others => '0');      
              
end Behavioral;
